@@ -15,7 +15,7 @@
     
     LICENCE
     
-    FMRIB Software Library, Release 4.0 (c) 2007, The University of
+    FMRIB Software Library, Release 5.0 (c) 2012, The University of
     Oxford (the "Software")
     
     The Software remains the property of the University of Oxford ("the
@@ -64,7 +64,7 @@
     interested in using the Software commercially, please contact Isis
     Innovation Limited ("Isis"), the technology transfer company of the
     University, to negotiate a licence. Contact details are:
-    innovation@isis.ox.ac.uk quoting reference DE/1112. */
+    innovation@isis.ox.ac.uk quoting reference DE/9564. */
 
 
 // Creates a mean time series (ignoring zeros) from the input 4D volume
@@ -83,7 +83,7 @@ using namespace Utilities;
 // The two strings below specify the title and example usage that is
 //  printed out as the help or usage message
 
-string title="fslmeants (Version 1.2)\nCopyright(c) 2004-2009, University of Oxford (Mark Jenkinson, Christian F. Beckmann)\nPrints average timeseries (intensities) to the screen (or saves to a file).\nThe average is taken over all voxels in the mask (or all voxels in the image if no mask is specified).\n";
+string title="fslmeants \nCopyright(c) 2004-2009, University of Oxford (Mark Jenkinson, Christian F. Beckmann)\nPrints average timeseries (intensities) to the screen (or saves to a file).\nThe average is taken over all voxels in the mask (or all voxels in the image if no mask is specified).\n";
 string examples="fslmeants -i filtered_func_data -o meants.txt -m my_mask\nfslmeants -i filtered_func_data -m my_mask\nfslmeants -i filtered_func_data -c 24 19 10";
 
 // Each (global) object below specificies as option and can be accessed
@@ -132,6 +132,10 @@ Option<int> order(string("--order"), 1,
 Option<bool> transpose(string("--transpose"), false,
 		  string("        output results in transpose format (one row per voxel/mean)"),
 		  false, no_argument);
+Option<bool> weightedMask(string("-w"), false,
+		  string("output weighted mean, using mask values as weights, and exit."),
+		  false, no_argument);
+
 
 int nonoptarg;
 
@@ -156,6 +160,7 @@ int main(int argc,char *argv[])
   options.add(transpose);
   options.add(verbose);
   options.add(help);
+  options.add(weightedMask);
   
   nonoptarg = options.parse_command_line(argc, argv);
   
@@ -182,6 +187,32 @@ int main(int argc,char *argv[])
     mask = 1.0;
   }
 
+
+
+  if ( weightedMask.value() ) {
+    Matrix meanwts(vin.tsize(),1);
+    for(int t=0;t<vin.tsize();t++) {    
+      double mean(0);
+      double meanWeight(0);
+      for (int z=mask.minz(); z<=mask.maxz(); z++) 
+	for (int y=mask.miny(); y<=mask.maxy(); y++) 
+	  for (int x=mask.minx(); x<=mask.maxx(); x++) {
+	    mean+=mask(x,y,z)*vin(x,y,z,t);	
+	    meanWeight+=mask(x,y,z);
+	  }
+      meanwts(t+1,1)=(mean/meanWeight);
+    }
+
+    if ( transpose.value() ) 
+      meanwts=meanwts.t();
+    if ( outmat.set() ) 
+      write_ascii_matrix(meanwts,outmat.value());
+    else 
+      cout << meanwts << endl;
+    return(0);
+  }
+
+
   int nlabs=1;
   if (labelname.set() && showall.unset()) {
     read_volume(label,labelname.value());
