@@ -15,7 +15,7 @@
     
     LICENCE
     
-    FMRIB Software Library, Release 4.0 (c) 2007, The University of
+    FMRIB Software Library, Release 5.0 (c) 2012, The University of
     Oxford (the "Software")
     
     The Software remains the property of the University of Oxford ("the
@@ -64,7 +64,7 @@
     interested in using the Software commercially, please contact Isis
     Innovation Limited ("Isis"), the technology transfer company of the
     University, to negotiate a licence. Contact details are:
-    innovation@isis.ox.ac.uk quoting reference DE/1112. */
+    innovation@isis.ox.ac.uk quoting reference DE/9564. */
 
 #include "easyoptions.h"
 #include "easylog.h"
@@ -80,9 +80,13 @@ class DataSet
 
   const NEWIMAGE::volume<float>& GetMask() const { return mask; }
   const NEWMAT::Matrix& GetVoxelData() const { return voxelData; }
+  const NEWMAT::Matrix& GetVoxelCoords() const { return voxelCoords; }
 
  protected:
   NEWIMAGE::volume<float> mask;
   NEWMAT::Matrix voxelData;
+
+  // coordinates of each voxel
+  NEWMAT::Matrix voxelCoords;
 };
 
