@@ -15,7 +15,7 @@
     
     LICENCE
     
-    FMRIB Software Library, Release 4.0 (c) 2007, The University of
+    FMRIB Software Library, Release 5.0 (c) 2012, The University of
     Oxford (the "Software")
     
     The Software remains the property of the University of Oxford ("the
@@ -64,7 +64,7 @@
     interested in using the Software commercially, please contact Isis
     Innovation Limited ("Isis"), the technology transfer company of the
     University, to negotiate a licence. Contact details are:
-    innovation@isis.ox.ac.uk quoting reference DE/1112. */
+    innovation@isis.ox.ac.uk quoting reference DE/9564. */
 
 #define _GNU_SOURCE 1
 #define POSIX_SOURCE 1
@@ -82,8 +82,8 @@ using namespace std;
 // The two strings below specify the title and example usage that is
 //  printed out as the help or usage message
 
-string title="midtrans (Version 2.0)\nCopyright(c) 2010, University of Oxford (Mark Jenkinson)";
-string examples="midtrans [options] transform1 transform2 ... transformN";
+string title="midtrans \nCopyright(c) 2010, University of Oxford (Mark Jenkinson)";
+string examples="midtrans [options] transform1 transform2 ... transformN\n  e.g. midtrans -o temp2mid.mat A2temp.mat B2temp.mat C2temp.mat\n       midtrans -o C2mid.mat A2C.mat B2C.mat ident.mat";
 
 // Each (global) object below specificies as option and can be accessed
 //  anywhere in this file (since they are global).  The order of the
@@ -105,7 +105,7 @@ Option<string> templte(string("--template"), string(""),
 		  string("input filename for template image (needed for fix origin)"),
 		  false, requires_argument);
 Option<string> separateouts(string("--separate"), string(""),
-		  string("basename for output of separate matrices (final name includes a number)"),
+		  string("basename for output of separate matrices (final name includes a number; e.g. img2mid0001.mat)"),
 		  false, requires_argument);
 Option<string> outname(string("-o,--out"), string(""),
 		  string("output filename for matrix"),
