@@ -3,9 +3,9 @@
     
     meldata.cc - data handler / container class
 
-    Christian F. Beckmann, FMRIB Image Analysis Group
+    Christian F. Beckmann, FMRIB Analysis Group
     
-    Copyright (C) 1999-2008 University of Oxford */
+    Copyright (C) 1999-2013 University of Oxford */
 
 /*  Part of FSL - FMRIB's Software Library
     http://www.fmrib.ox.ac.uk/fsl
@@ -75,6 +75,7 @@
 #include "melodic.h" 
 #include "utils/log.h"
 #include <time.h>
+#include <algorithm>
 #include "miscmaths/miscprob.h"
 #include "melhlprfns.h" 
 
@@ -86,38 +87,34 @@ namespace Melodic{
 
   ReturnMatrix MelodicData::process_file(string fname, int numfiles)
   {
+	dbgmsg(string("START: process_file") << endl);	
+	
 	Matrix tmpData;
 	{
     	volume4D<float> RawData;
 
-		if(opts.debug.value())
-			memmsg(" before reading file "<< fname);
+		memmsg(" before reading file "<< fname);
 
     	//read data
     	message("Reading data file " << fname << "  ... ");
     	read_volume4D(RawData,fname);
     	message(" done" << endl);
-    	if(opts.debug.value())
-			memmsg(" after reading file "<< fname);
+		memmsg(" after reading file "<< fname);
 		
 		del_vols(RawData,opts.dummy.value());
     
     	Mean += meanvol(RawData)/numfiles;
 
 		//estimate smoothness
-		if(opts.debug.value())
-			memmsg(" before est smoothness ");
+		memmsg(" before est smoothness ");
     	if((Resels == 0)&&(!opts.filtermode))
       		Resels = est_resels(RawData,Mask);
-		if(opts.debug.value())
-			memmsg(" after smoothness ");
+		memmsg(" after smoothness ");
 	
     	//reshape
-		if(opts.debug.value())
-			memmsg(" before reshape ");
+		memmsg(" before reshape ");
     	tmpData = RawData.matrix(Mask);
-    	if(opts.debug.value())
-			memmsg(" after reshape ");	  
+		memmsg(" after reshape ");	  
 	}    
         
     //convert to percent BOLD signal change
@@ -131,11 +128,9 @@ namespace Melodic{
 		if(opts.remove_meanvol.value())
 		{	      
 			message(string("  Removing mean image ..."));
-      		if(opts.debug.value())
-				memmsg(" before remmean ");
+			memmsg(" before remmean ");
       		remmean(tmpData,meanR,1);
-      		if(opts.debug.value())
-				memmsg(" after remmean ");
+			memmsg(" after remmean ");
       		message(" done" << endl);
 		}
 		else meanR=ones(1,tmpData.Ncols());
@@ -164,23 +159,24 @@ namespace Melodic{
     }
       
     //variance - normalisation
-   	if(opts.debug.value())
-		memmsg(" before VN ");
+
     if(opts.varnorm.value()){
+	  memmsg(" before VN ");
       message("  Normalising by voxel-wise variance ..."); 
-			if(stdDev.Storage()==0)
+		outMsize("stdDev",stdDev);
+//			if(stdDev.Storage()==0)
       	stdDev = varnorm(tmpData,std::min(30,tmpData.Nrows()-1),
-					opts.vn_level.value())/numfiles;
-			else 	
-				stdDev += varnorm(tmpData,std::min(30,tmpData.Nrows()-1),
-					opts.vn_level.value())/numfiles;
+					opts.vn_level.value(), opts.econ.value());
+//			else 	
+//				stdDev += varnorm(tmpData,std::min(30,tmpData.Nrows()-1),
+//					opts.vn_level.value(), opts.econ.value())/numfiles;
       stdDevi = pow(stdDev,-1); 
-	  if(opts.debug.value())
-	    memmsg(" in VN ");
+	  memmsg(" in VN ");
       message(" done" << endl);
     }
 
 	tmpData.Release();
+	dbgmsg(string("END: process_file") << endl);	
     return tmpData;
   }
 
@@ -219,95 +215,160 @@ namespace Melodic{
     return tmp;
   }
 
-  void MelodicData::set_TSmode()
+  void MelodicData::set_TSmode_depr()
   {
-   	dbgmsg(string("START: set_TSmode"));	
-	
-	Matrix tmp, tmpT, tmpS, tmpT2, tmpS2, tmpT3;
-	
-    tmp = expand_dimred(mixMatrix);
-    tmpT = zeros(tmp.Nrows()/numfiles, tmp.Ncols());
-    tmpS = ones(numfiles, tmp.Ncols());
+		Matrix tmp, tmpT, tmpS, tmpT2, tmpS2, tmpT3;
 
-	outMsize("tmp",tmp);
-	outMsize("tmpT",tmpT);
-	outMsize("tmpS",tmpS);
+	    tmp = expand_dimred(mixMatrix);
+	    tmpT = zeros(tmp.Nrows()/numfiles, tmp.Ncols());
+	    tmpS = ones(numfiles, tmp.Ncols());
 
-  	dbgmsg(string("   approach ") << opts.approach.value() << endl);	
+		outMsize("tmp",tmp);
+		outMsize("tmpT",tmpT);
+		outMsize("tmpS",tmpS);
 
-	if(opts.approach.value()==string("tica")){
-      message("Calculating T- and S-modes " << endl);
-      explained_var = krfact(tmp,tmpT,tmpS);
-      Tmodes.clear(); Smodes.clear();
-      for(int ctr = 1; ctr <= tmp.Ncols(); ctr++){
-	    tmpT3 << reshape(tmp.Column(ctr),tmpT.Nrows(),numfiles);
-		outMsize("tmpT3", tmpT3);
-      	tmpT2 << tmpT.Column(ctr);
-      	tmpS2 << tmpS.Column(ctr);
-		tmpT3 << SP(tmpT3,pow(ones(tmpT3.Nrows(),1)*tmpS2.t(),-1));
-		if(numfiles>1)
-	      tmpT2 |= tmpT3;
-		if(mean(tmpS2,1).AsScalar()<0){
-		  tmpT2*=-1.0;
-		  tmpS2*=-1.0;
+	  	dbgmsg(string("   approach ") << opts.approach.value() << endl);	
+
+		if(opts.approach.value()==string("tica")){
+	      message("Calculating T- and S-modes " << endl);
+	      explained_var = krfact(tmp,tmpT,tmpS);
+	      Tmodes.clear(); Smodes.clear();
+	      for(int ctr = 1; ctr <= tmp.Ncols(); ctr++){
+		    tmpT3 << reshape(tmp.Column(ctr),tmpT.Nrows(),numfiles);
+			outMsize("tmpT3", tmpT3);
+	      	tmpT2 << tmpT.Column(ctr);
+	      	tmpS2 << tmpS.Column(ctr);
+			tmpT3 << SP(tmpT3,pow(ones(tmpT3.Nrows(),1)*tmpS2.t(),-1));
+			if(numfiles>1)
+		      tmpT2 |= tmpT3;
+			if(mean(tmpS2,1).AsScalar()<0){
+			  tmpT2*=-1.0;
+			  tmpS2*=-1.0;
+			}
+	      	add_Tmodes(tmpT2);
+	      	add_Smodes(tmpS2);
+		  }
 		}
-      	add_Tmodes(tmpT2);
-      	add_Smodes(tmpS2);
-	  }
-	}
-	else{
-		Tmodes.clear();
-		Smodes.clear();
-		for(int ctr = 1; ctr <= tmp.Ncols(); ctr++){
-			tmpT3 << tmp.Column(ctr);
-			add_Tmodes(tmpT3);
+		else{
+			Tmodes.clear();
+			Smodes.clear();
+			for(int ctr = 1; ctr <= tmp.Ncols(); ctr++){
+				tmpT3 << tmp.Column(ctr);
+				add_Tmodes(tmpT3);
+			}
 		}
-	}
-	
-    if(opts.approach.value()!=string("concat")){
-	  //add GLM OLS fit
-	  dbgmsg(string(" GLM fitting ") << endl);
 
-	  if(Tdes.Storage()){
-	    Matrix alltcs = Tmodes.at(0).Column(1);
-	    for(int ctr=1; ctr < (int)Tmodes.size();ctr++)
-		  alltcs|=Tmodes.at(ctr).Column(1);
-	    if((alltcs.Nrows()==Tdes.Nrows())&&(Tdes.Nrows()>Tdes.Ncols()))
-		  glmT.olsfit(alltcs,Tdes,Tcon);
-	  }
-	  if(Sdes.Storage()){
-	    Matrix alltcs = Smodes.at(0);
-	    for(int ctr=1; ctr < (int)Smodes.size();ctr++)
-	  	  alltcs|=Smodes.at(ctr);
-	    if((alltcs.Nrows()==Sdes.Nrows())&&(Sdes.Nrows()>Sdes.Ncols()&&alltcs.Nrows()>2))
-		  glmS.olsfit(alltcs,Sdes,Scon);
-	  }
-		
-    }
-  //    else{
-//		dbgmsg(string(" Bypassing krfac ") << endl);
-//        add_Tmodes(tmp);
-//		add_Smodes(tmpS);
-//      }
-	
-	dbgmsg(string("END: set_TSmode"));	
+	    if(opts.approach.value()!=string("concat")){
+		  //add GLM OLS fit
+		  dbgmsg(string(" GLM fitting ") << endl);
+
+		  if(Tdes.Storage()){
+		    Matrix alltcs = Tmodes.at(0).Column(1);
+		    for(int ctr=1; ctr < (int)Tmodes.size();ctr++)
+			  alltcs|=Tmodes.at(ctr).Column(1);
+		    if((alltcs.Nrows()==Tdes.Nrows())&&(Tdes.Nrows()>Tdes.Ncols()))
+			  glmT.olsfit(alltcs,Tdes,Tcon);
+		  }
+		  if(Sdes.Storage()){
+		    Matrix alltcs = Smodes.at(0);
+		    for(int ctr=1; ctr < (int)Smodes.size();ctr++)
+		  	  alltcs|=Smodes.at(ctr);
+		    if((alltcs.Nrows()==Sdes.Nrows())&&(Sdes.Nrows()>Sdes.Ncols()&&alltcs.Nrows()>2))
+			  glmS.olsfit(alltcs,Sdes,Scon);
+		  }
+
+	    }
+	  //    else{
+	//		dbgmsg(string(" Bypassing krfac ") << endl);
+	//        add_Tmodes(tmp);
+	//		add_Smodes(tmpS);
+	//      }
   }
 
-  void MelodicData::setup()
-  { 
-    numfiles = (int)opts.inputfname.value().size();
-    setup_misc();
-	if(opts.debug.value())
-		memmsg(" after setup_misc ");
-		
-	if(opts.filtermode){ // basic setup for filtering only
-		Data = process_file(opts.inputfname.value().at(0));
+  void MelodicData::dual_regression()
+  {
+	dbgmsg(string("START: dual_regression") << endl);	
+
+	Tmodes.clear();
+	Smodes.clear();
+
+	bool tmpvarnorm = opts.varnorm.value();
+	// Switch off variance normalisation
+	opts.varnorm.set_T(false);
+
+	Log drO;
+
+	if(opts.dr_out.value())
+		drO.makeDir(logger.appendDir("dr"),"dr.log");
+
+	Matrix tmpcont = diag(ones(IC.Nrows(),1)), s1,s2, tmpData, alltcs;
+	basicGLM tmpglm;
+	for(int ctr = 0; ctr < numfiles; ctr++){
+		tmpData = process_file(opts.inputfname.value().at(ctr), numfiles);
+		//may want to remove the spatial means first
+		tmpglm.olsfit(remmean(tmpData.t(),1),remmean(IC.t(),1),tmpcont);
+		s1=tmpglm.get_beta().t();
+
+		outMsize("s1",s1);
+	    outMsize("alltcs",alltcs);
+		if(alltcs.Storage()==0)
+			alltcs=s1;
+		else
+			alltcs&=s1;
+			
+		// output DR
+		if(opts.dr_out.value()){
+			
+			dbgmsg(string("START: dual_regression output") << endl);
+			write_ascii_matrix(drO.appendDir("dr_stage1_subject"+num2str(ctr,4)+".txt"),s1);
+			//des_norm
+			s1 =  SP(s1,ones(s1.Nrows(),1)*pow(stdev(s1,1),-1));
+			tmpglm.olsfit(remmean(tmpData),remmean(s1,1),tmpcont);
+			s2=tmpglm.get_beta();
+			save4D(s2,string("dr/dr_stage2_subject"+num2str(ctr,4)));
+			s2=tmpglm.get_z();
+			save4D(s2,string("dr/dr_stage2_subject"+num2str(ctr,4)+"_Z"));
+		}
+    }
+
+	for(int ctr = 1; ctr <= alltcs.Ncols(); ctr++){
+		tmpcont << alltcs.Column(ctr);
+		add_Tmodes(tmpcont);
+	}
+	
+	for(int ctrC = 1; ctrC <=IC.Nrows(); ctrC++){
+		Matrix tmpall = zeros(numfiles,IC.Ncols());
+		string fnout = string("dr/dr_stage2_ic"+num2str(ctrC-1,4));
+		for(int ctrS = 0; ctrS < numfiles; ctrS++){
+			string fnin = logger.appendDir(string("dr/dr_stage2_subject"+num2str(ctrS,4)));
+			dbgmsg(fnout << endl << fnin << endl);
+			volume4D<float> vol;
+			read_volume4DROI(vol,fnin,0,0,0,ctrC-1,-1,-1,-1,ctrC-1);
+			
+			Matrix tmp2 = vol.matrix(Mask);
+		    tmpall.Row(ctrS+1) << vol.matrix(Mask);
+		}
+		save4D(tmpall,fnout);
 	}
-	else{
 
-      	if((numfiles > 1) && (opts.approach.value()==string("defl") || opts.approach.value()==string("symm")))
-        	opts.approach.set_T("tica");
+    opts.varnorm.set_T(tmpvarnorm);	
+	dbgmsg(string("END: dual_regression") << endl);	
+  }
 
+  void MelodicData::set_TSmode()
+  {
+   	dbgmsg(string("START: set_TSmode")<< endl);	
+	if(opts.dr.value())
+		dual_regression();
+	else
+		set_TSmode_depr();
+	
+	dbgmsg(string("END: set_TSmode")<< endl);	
+  }
+
+  void MelodicData::setup_classic()
+  {
+	    dbgmsg(string("START: setup_classic") << endl);	
     	Matrix alldat, tmpData;
 		bool tmpvarnorm = opts.varnorm.value();
 
@@ -315,28 +376,23 @@ namespace Melodic{
 			opts.varnorm.set_T(false);
 		}
     	alldat = process_file(opts.inputfname.value().at(0), numfiles) / numfiles;
-		if(opts.debug.value())
-			memmsg(" after process_file ");
-			
+		memmsg(" after process_file ");
+
 		if(opts.pca_dim.value() > alldat.Nrows()-2){
 			cerr << "ERROR:: too many components selected \n\n";
 			exit(2);
 		}
-		
- 		if(opts.debug.value())
-			save4D(alldat,string("preproc_dat") + num2str(1));   
+
 		for(int ctr = 1; ctr < numfiles; ctr++){
     		tmpData = process_file(opts.inputfname.value().at(ctr), numfiles) / numfiles;
-			if(opts.debug.value())
-				save4D(tmpData,string("preproc_dat") + num2str(ctr+1));
 			if(tmpData.Ncols() == alldat.Ncols() && tmpData.Nrows() == alldat.Nrows())
-      			alldat += tmpData;	
+      			alldat = alldat + tmpData;	
 			else{
 				    if(opts.approach.value()==string("tica")){
 						cerr << "ERROR:: data dimensions do not match, TICA not possible \n\n";
 						exit(2); 
 					}
-					
+
 					if(tmpData.Ncols() == alldat.Ncols()){
 						int mindim = min(alldat.Nrows(),tmpData.Nrows());
 						alldat = alldat.Rows(1,mindim);
@@ -345,7 +401,7 @@ namespace Melodic{
 					}				
 					else 	
 					message("Data dimensions do not match - ignoring "+opts.inputfname.value().at(ctr) << endl);
-				}
+			}
    		}	
 
     	//update mask
@@ -358,11 +414,11 @@ namespace Melodic{
 		if((numfiles > 1 ) && opts.joined_vn.value() && tmpvarnorm){	
 			//variance - normalisation
 	    	message(endl<<"Normalising jointly by voxel-wise variance ..."); 
-	    	stdDev = varnorm(alldat,alldat.Nrows(),3.1);
+	    	stdDev = varnorm(alldat,alldat.Nrows(),opts.vn_level.value(),opts.econ.value());
 	    	stdDevi = pow(stdDev,-1); 
 	    	message(" done" << endl);
 		}
-		
+
 		if(numfiles>1)
     		message(endl << "Initial data size : "<<alldat.Nrows()<<" x "<<alldat.Ncols()<<endl<<endl);
 
@@ -393,7 +449,7 @@ namespace Melodic{
 			Matrix tmpPscales;
 			tmpPscales = param.t() * alldat;
 			paramS = stdev(tmpPscales.t());
-		    
+
     		calc_white(pcaE, pcaD, order, param, paramS, whiteMatrix, dewhiteMatrix);
 		}else
     		calc_white(pcaE, pcaD, order, whiteMatrix, dewhiteMatrix);
@@ -418,11 +474,12 @@ namespace Melodic{
       		WM.push_back(tmp);
     	} 
 		else {
-  
-		dbgmsg("Multi-Subject ICA");
-  		for(int ctr = 0; ctr < numfiles; ctr++){
+
+			dbgmsg("Multi-Subject ICA");
+	        	//stdDev.CleanUp();
+  			for(int ctr = 0; ctr < numfiles; ctr++){
 				tmpData = process_file(opts.inputfname.value().at(ctr), numfiles);
-	
+
 				if(opts.joined_vn.value() && tmpvarnorm){
 					dbgmsg("tmpData normalisation"<< endl);
 					dbgmsg("stdDev "  << stdDev(1,2)<< endl);
@@ -433,11 +490,11 @@ namespace Melodic{
 				Matrix newWM,newDWM; 
 				if(!opts.joined_whiten.value()){	  
 					message("  Individual whitening in a " << order << " dimensional subspace " << endl);
-      	  			std_pca(tmpData, RXweight, Corr, pcaE, pcaD);
+      	  			std_pca(tmpData, RXweight, Corr, pcaE, pcaD, opts.econ.value());
 	  				calc_white(pcaE, pcaD, order, newWM, newDWM);
 				}else{
 					if(!opts.dr_pca.value()){
-						std_pca(whiteMatrix*tmpData, RXweight, Corr, pcaE, pcaD);
+						std_pca(whiteMatrix*tmpData, RXweight, Corr, pcaE, pcaD, opts.econ.value());
 						calc_white(pcaE, pcaD, order, newWM, newDWM);		
 						newDWM=(dewhiteMatrix*newDWM);
 						newWM=(newWM*whiteMatrix);
@@ -450,7 +507,7 @@ namespace Melodic{
 						remmean(tmp1,2);
 						tmp1 *= tmpData.t();
 						tmp2 = pinv(tmp1.t()).t();  
-						std_pca(tmp1 * tmpData, RXweight, Corr, pcaE, pcaD);
+						std_pca(tmp1 * tmpData, RXweight, Corr, pcaE, pcaD, opts.econ.value());
 						calc_white(pcaE, pcaD, order, newWM, newDWM);		
 						newDWM=(tmp2*newDWM);
 						newWM=(newWM * tmp1);
@@ -459,31 +516,124 @@ namespace Melodic{
 				DWM.push_back(newDWM);
 				WM.push_back(newWM);
 				tmpData = newWM * tmpData;
-				
+
 				//concatenate Data
 				if(Data.Storage() == 0)
 	  			Data = tmpData;
 				else
 	  			Data &= tmpData;
-      		}
+      	    }
         }
-		opts.varnorm.set_T(tmpvarnorm);
+    	opts.varnorm.set_T(tmpvarnorm);
+   	    dbgmsg(string("END: setup_classic") << endl);	
+ 
+  }
 
-    	message(endl << "  Data size : "<<Data.Nrows()<<" x "<<Data.Ncols()<<endl<<endl);
- 		outMsize("stdDev",stdDev);
-   
-    	//meanC=mean(Data,2);
+  void MelodicData::setup_migp()
+  {
+    dbgmsg(string("START: setup_migp") << endl);	
+	
+	std::vector<int> myctr;
+	for (int i=0; i< numfiles ; ++i) myctr.push_back(i); 
+
+	if(opts.migp_shuffle.value()){
+		message("Randomising input file order" << endl);
+		std::random_shuffle ( myctr.begin(), myctr.end() );
+	}
+	
+	Matrix tmpData;
+	bool tmpvarnorm = opts.varnorm.value();
+
+	if(numfiles > 1 && opts.joined_vn.value()){
+		opts.varnorm.set_T(false);
+	}
+		
+	for(int ctr = 0; ctr < numfiles; ctr++){
+		tmpData = process_file(opts.inputfname.value().at(myctr.at(ctr)), numfiles) / numfiles;
+		
+		if (opts.migpN.value()==0){
+			opts.migpN.set_T(2*tmpData.Nrows()-1);
+		}
 		if(opts.debug.value())
-			save4D(Data,"concat_data");    
-    	//save the mean & mask
-    	save_volume(Mask,logger.appendDir("mask"));
-    	save_volume(Mean,logger.appendDir("mean"));
+			save4D(tmpData,string("preproc_dat") + num2str(ctr+1));
+			
+		if(Data.Storage()==0)
+			Data = tmpData;
+		else
+  			Data &= tmpData;
+
+		outMsize("Data", Data);
+		//reduce dim down to manageable level
+		if(Data.Nrows() > opts.migp_factor.value()*opts.migpN.value() || ctr==numfiles-1){
+			message("  Reducing data matrix to a  " << opt.migpN.value() << " dimensional subspace " << endl);
+			Matrix pcaE, Corr;
+			RowVector pcaD;
+			std_pca(Data, RXweight, Corr, pcaE, pcaD, opts.econ.value());
+		    pcaE = pcaE.Columns(pcaE.Ncols()-opts.migpN.value()+1,pcaE.Ncols());
+		    Data = pcaE.t() * Data;	
+		}
+		outMsize("Data", Data);
+		
     }
+
+  	//update mask
+    if(opts.update_mask.value()){
+      message(endl<< "Excluding voxels with constant value ...");
+      update_mask(Mask, Data); 
+      message(" done" << endl);
+    }
+
+	Matrix tmp = IdentityMatrix(Data.Nrows());
+	DWM.push_back(tmp);
+	WM.push_back(tmp);
+   	opts.varnorm.set_T(tmpvarnorm);
+
+	if(opts.varnorm2.value()){
+	  message("  Normalising by voxel-wise variance ..."); 
+      stdDev = varnorm(Data,std::min(30,Data.Nrows()-1),
+					opts.vn_level.value(), opts.econ.value());
+	  message(" done" << endl);
+	}
+	
+    dbgmsg(string("END: setup_migp") << endl);	
+  }
+
+  void MelodicData::setup()
+  { 
+	dbgmsg(string("START: setup") << endl);	
+		
+	numfiles = (int)opts.inputfname.value().size();
+    setup_misc();
+	if(opts.debug.value())
+		memmsg(" after setup_misc ");
+		
+	if(opts.filtermode){ // basic setup for filtering only
+		Data = process_file(opts.inputfname.value().at(0));
+	}
+	else{
+		if((numfiles > 1) && (opts.approach.value()==string("defl") || opts.approach.value()==string("symm")))
+	       opts.approach.set_T("concat");
+		if(opts.migp.value())
+			setup_migp();
+		else
+			setup_classic();   
+    }
+
+    message(endl << "  Data size : "<<Data.Nrows()<<" x "<<Data.Ncols()<<endl<<endl);
+	outMsize("stdDev",stdDev);
+	//meanC=mean(Data,2);
+	if(opts.debug.value())
+		save4D(Data,"concat_data");    
+    //save the mean & mask
+    save_volume(Mask,logger.appendDir("mask"));
+    save_volume(Mean,logger.appendDir("mean"));
+
+	dbgmsg(string("END: setup") << endl);	    
   } // void setup()
 	
-
   void MelodicData::setup_misc()
   {
+	dbgmsg(string("START: setup_misc") << endl);	
 
     //initialize Mean
     read_volume(Mean,opts.inputfname.value().at(0));
@@ -561,9 +711,11 @@ namespace Melodic{
 		}
 	}
 	remmean(Tdes);
+	
+	dbgmsg(string("END: setup_misc") << endl);	
+    
   }
 
-
   void MelodicData::save()
   {   
 
@@ -620,7 +772,7 @@ namespace Melodic{
 				}
 
 				if(opts.output_origIC.value())
-	  			save4D(stdNoisei,string("Noise_stddev_inv"));
+	  			save4D(stdNoisei,string("Noise__inv"));
       }
      
     //Output T- & S-modes
@@ -683,7 +835,6 @@ namespace Melodic{
 		message("...done" << endl);
   } //void save()
 
-
   int MelodicData::remove_components()
   {  
     message("Reading " << opts.filtermix.value() << endl) 
@@ -758,7 +909,6 @@ namespace Melodic{
     return 0;
   } // int remove_components()
 
-
   void MelodicData::create_RXweight()
   {
     message("Reading the weights for the covariance R_X from file "<< opts.segment.value() << endl);
@@ -768,7 +918,6 @@ namespace Melodic{
     RXweight = tmpRX.matrix(Mask);
   } 
 
-
   void MelodicData::est_smoothness()
   {
     if(Resels == 0){
@@ -805,7 +954,6 @@ namespace Melodic{
     }
   }
 
-
   unsigned long MelodicData::standardise(volume<float>& mask, volume4D<float>& R)
   {
     	unsigned long count = 0;
@@ -847,7 +995,6 @@ namespace Melodic{
     	return count;
   }
 
-
   float MelodicData::est_resels(volume4D<float> R, volume<float> mask)
   {
     message("  Estimating data smoothness ... ");
@@ -861,7 +1008,7 @@ namespace Melodic{
     if (R.zsize() <= 1) { usez = false; }
 
     enum {X = 0, Y, Z};
-    float SSminus[3] = {0, 0, 0}, S2[3] = {0, 0, 0};
+    double SSminus[3] = {0, 0, 0}, S2[3] = {0, 0, 0};
 
     int zstart=1;
     if (!usez) zstart=0;
@@ -891,8 +1038,8 @@ namespace Melodic{
 	    				}
 	  			}
 
-    float norm = 1.0/(float) N;
-    float v = dof;	// v - degrees of freedom (nu)  
+    double norm = 1.0/(double) N;
+    double v = dof;	// v - degrees of freedom (nu)  
     if(R.tsize() > 1) {
       norm = (v - 2) / ((v - 1) * N * R.tsize());
     }
@@ -906,24 +1053,23 @@ namespace Melodic{
       if (SSminus[Z]>=0.99999999*S2[Z]) 
 				SSminus[Z]=0.99999*S2[Z];
     // Convert to sigma squared
-    float sigmasq[3] = {0,0,0};
+    double sigmasq[3] = {0,0,0};
     sigmasq[X] = -1.0 / (4 * log(fabs(SSminus[X]/S2[X])));
     sigmasq[Y] = -1.0 / (4 * log(fabs(SSminus[Y]/S2[Y])));
     if (usez) { sigmasq[Z] = -1.0 / (4 * log(fabs(SSminus[Z]/S2[Z]))); }
     
     // Convert to full width half maximum
-    float FWHM[3] = {0,0,0};
+    double FWHM[3] = {0,0,0};
     FWHM[X] = sqrt(8 * log(2) * sigmasq[X]);
     FWHM[Y] = sqrt(8 * log(2) * sigmasq[Y]);
     if (usez) { FWHM[Z] = sqrt(8 * log(2) * sigmasq[Z]); }
-    float resels = FWHM[X] * FWHM[Y];
+    double resels = FWHM[X] * FWHM[Y];
     if (usez) resels *= FWHM[Z];
 
     message(" done " <<endl);
     return resels;
   }
 
-
   void MelodicData::create_mask(volume<float>& theMask)
   {
     if(opts.use_mask.value() && opts.maskfname.value().size()>0){   // mask provided 
@@ -998,7 +1144,6 @@ namespace Melodic{
     }
   } //void create_mask()
 
-
   void MelodicData::sort()
   {
     int numComp = mixMatrix.Ncols(), numVox = IC.Ncols(), 
@@ -1082,13 +1227,6 @@ namespace Melodic{
     unmixMatrix = pinv(mixMatrix);
   }
 
-  void MelodicData::reregress()
-  {
-	if((numfiles > 1)){
-	   
-	}
-  }
-  
   void MelodicData::status(const string &txt)
   {
     cout << "MelodicData Object " << txt << endl;
