@@ -3,9 +3,9 @@
     
     meldata.h - data container class
 
-    Christian F. Beckmann, FMRIB Image Analysis Group
+    Christian F. Beckmann, FMRIB Analysis Group
     
-    Copyright (C) 1999-2008 University of Oxford */
+    Copyright (C) 1999-2013 University of Oxford */
 
 /*  Part of FSL - FMRIB's Software Library
     http://www.fmrib.ox.ac.uk/fsl
@@ -116,7 +116,11 @@ namespace Melodic{
       }
 
       int  remove_components();
+
+      void setup_classic();
+      void setup_migp();
       void setup();
+
       void status(const string &txt);
 
       inline Matrix& get_pcaE() {return pcaE;}
@@ -159,6 +163,7 @@ namespace Melodic{
 			}
       }
 
+      void set_TSmode_depr();	
       void set_TSmode();
 
       inline Matrix& get_param() {return param;} 
@@ -262,7 +267,7 @@ namespace Melodic{
       }
       
       void sort();
-	  void reregress();
+	  void dual_regression();
 
       vector<Matrix> DWM, WM;
 			basicGLM glmT, glmS;
