@@ -3,9 +3,9 @@
     
     meloptions.h - class for command line options
 
-    Christian F. Beckmann, FMRIB Image Analysis Group
+    Christian F. Beckmann, FMRIB Analysis Group
     
-    Copyright (C) 1999-2008 University of Oxford */
+    Copyright (C) 1999-2013 University of Oxford */
 
 /*  Part of FSL - FMRIB's Software Library
     http://www.fmrib.ox.ac.uk/fsl
@@ -114,12 +114,19 @@ class MelodicOptions {
   	Option<bool>   joined_whiten;
   	Option<bool>   joined_vn;
 	Option<bool>   dr_pca;
-	Option<float>	 vn_level;
+	Option<bool>   migp;
+	Option<int>    migpN;
+	Option<bool>   migp_shuffle;
+	Option<int>	   migp_factor;
+	Option<bool>   dr;
+	Option<bool>   dr_out;
+	Option<float>  vn_level;
   	Option<int>    numICs;
   	Option<string> approach;
   	Option<string> nonlinearity;
 
   	Option<bool>   varnorm;
+ 	Option<bool>   varnorm2;
   	Option<bool>   pbsc;
   	Option<bool>   pspec;
   	Option<string> segment;
@@ -138,21 +145,21 @@ class MelodicOptions {
   	Option<string> filter; 
 
   	Option<bool>   genreport;
-		Option<string> guireport;
-		Option<string> bgimage;
+	Option<string> guireport;
+	Option<string> bgimage;
   	Option<float>  tr;
   	Option<bool>   logPower;
-		Option<bool>   addsigchng;
-		Option<bool>   allPPCA;
-		Option<bool>   varplots;
-		Option<bool>   varvals;
+	Option<bool>   addsigchng;
+	Option<bool>   allPPCA;
+	Option<bool>   varplots;
+	Option<bool>   varvals;
 
-		Option<string> fn_Tdesign;
-		Option<string> fn_Tcon;
-		Option<string> fn_TconF;
-		Option<string> fn_Sdesign;
-		Option<string> fn_Scon;	
-		Option<string> fn_SconF;	
+	Option<string> fn_Tdesign;
+	Option<string> fn_Tcon;
+	Option<string> fn_TconF;
+	Option<string> fn_Sdesign;
+	Option<string> fn_Scon;	
+	Option<string> fn_SconF;	
 	
   	Option<bool>   output_all;
   	Option<bool>   output_unmix;
@@ -187,7 +194,8 @@ class MelodicOptions {
   	Option<bool> guess_remderiv;
   	Option<bool> temporal;
 
-  	int retrystep;
+  	Option<float> retryfactor;
+  	Option<int> econ;
 
   	void parse_command_line(int argc, char** argv, Log& logger,  const string &p_version);
 
@@ -249,11 +257,29 @@ class MelodicOptions {
 	   string("switch on separate whitening"), 
 	   false, no_argument, false),
    joined_vn(string("--sep_vn"), true,
-   	   string("switch on separate variance nomalisation"), 
-       false, no_argument, false),
+   	   string("switch on separate variance nomalisation (as opposed to separate VN)"), 
+       false, no_argument),
    dr_pca(string("--mod_pca"), true,
 	   string("switch off modified PCA for concat ICA"),
 	   false, no_argument, false),
+   migp(string("--migp"), false,
+	   string("switch on MIGP data reduction"),
+	   false, no_argument, false),	
+   migpN(string("--migpN"), 0,
+	   string("Number of internal Eigenmaps"),
+	   false, requires_argument, false),
+   migp_shuffle(string("--migp_shuffle"), true,
+	   string("Randomise MIGP file order (default: TRUE)"),
+	   false, no_argument, false),
+   migp_factor(string("--migp_factor"), 2,
+	   string("Internal Factor of mem-threshold relative to number of Eigenmaps (default: 2)"),
+       false, requires_argument, false),
+   dr(string("--dr"), false,
+	   string("Dual Regression (default: false)"),
+	   false, no_argument, false),
+   dr_out(string("--dr_out"), false,
+	   string("Dual Regression output for MIGP/concat ICA"),
+	   false, no_argument, false),	
    vn_level(string("--vn_level"), float(2.3),
 	   string("variance nomalisation threshold level (Z> value is ignored)"), 
 	   false, requires_argument, false),
@@ -261,7 +287,7 @@ class MelodicOptions {
 	   string("numer of IC's to extract (for deflation approach)"), 
 	   false, requires_argument),
    approach(string("-a,--approach"),  string("symm"),
-	   string("approach for decomposition, 2D: defl, symm (default), 3D: tica (default), concat"),
+	   string("approach for decomposition, 2D: defl, symm (default), 3D: tica, concat (default)"),
 	   false, requires_argument),
    nonlinearity(string("--nl"), string("tanh"),
 	   string("\tnonlinearity: gauss, tanh, pow3, pow4"), 
@@ -269,6 +295,9 @@ class MelodicOptions {
    varnorm(string("--vn,--varnorm"), true,
 	   string("switch off variance normalisation"), 
 	   false, no_argument),
+   varnorm2(string("--vn2"), true,
+		string("switch off 2nd level variance normalisation"), 
+		false, no_argument, false),
    pbsc(string("--pbsc"), false,
 	   string("        switch on conversion to percent BOLD signal change"), 
 	   false, no_argument, false),
@@ -281,16 +310,16 @@ class MelodicOptions {
    tsmooth(string("--spca"),  false,
 	   string("smooth the eigenvectors prior to IC decomposition"), 
 	    false, no_argument, false),
-   epsilon(string("--eps,--epsilon"), 0.00005,
+   epsilon(string("--eps"), 0.00005,
 	   string("minimum error change"), 
 	   false, requires_argument),
-   epsilonS(string("--epsS,--epsilonS"), 0.03,
+   epsilonS(string("--epsS"), 0.03,
 	   string("minimum error change for rank-1 approximation in TICA"), 
 	   false, requires_argument),
    maxNumItt(string("--maxit"),  500,
 	   string("\tmaximum number of iterations before restart"), 
 	   false, requires_argument),
-   maxRestart(string("--maxrestart"),  6,
+   maxRestart(string("--maxrestart"),  -1,
 	   string("maximum number of restarts\n"), 
 	   false, requires_argument),
    rank1interval(string("--rank1interval"),  10,
@@ -303,16 +332,16 @@ class MelodicOptions {
 	   string("\tswitch off mixture modelling on IC maps\n "), 
 	   false, no_argument),
    ICsfname(string("--ICs"), string(""),
-	   string("\tfilename of the IC components file for mixture modelling"), 
+	   string("\tinput filename of the IC components file for mixture modelling"), 
 	   false, requires_argument),
    filtermix(string("--mix"),  string(""),
-	   string("\tmixing matrix for mixture modelling / filtering"), 
+	   string("\tinput filename of mixing matrix for mixture modelling / filtering"), 
 	   false, requires_argument),
    smodename(string("--smode"),  string(""),
-	   string("\tmatrix of session modes for report generation"), 
+	   string("\tinput filename of matrix of session modes for report generation"), 
 	   false, requires_argument),
    filter(string("-f,--filter"),  string(""),
-	   string("component numbers to remove\n "), 
+	   string("list of component numbers to remove\n "), 
 	   false, requires_argument),
    genreport(string("--report"), false,
 	   string("generate Melodic web report"), 
@@ -440,7 +469,12 @@ class MelodicOptions {
    temporal(string("--temporal"),  false,
 	   string("perform temporal ICA"), 
 	   false, no_argument, false),
-   retrystep(3),
+   retryfactor(string("--retryfactor"), float(0.95),
+		string("multiplicative factor for determining new dim if estimated dim fails to converge"),
+		false, requires_argument, false),
+   econ(string("--econ"), 20000, 
+	   string("set ctrl parameter for helperfns econ mode"),
+       false, requires_argument, false),
    options(title, usageexmpl)
    {
      try {  
@@ -458,11 +492,18 @@ class MelodicOptions {
 	    options.add(joined_whiten);
 	    options.add(joined_vn);
 		options.add(dr_pca);
+		options.add(migp);
+		options.add(migpN);
+		options.add(migp_shuffle);
+		options.add(migp_factor);
+		options.add(dr);
+		options.add(dr_out);
 	    options.add(vn_level);
 	    options.add(numICs);
 	    options.add(approach);
 	    options.add(nonlinearity);
 	    options.add(varnorm);
+		options.add(varnorm2);
 	    options.add(pbsc);
 	    options.add(pspec);
 	    options.add(segment);
@@ -521,6 +562,8 @@ class MelodicOptions {
 	    options.add(rescale_nht);
 	    options.add(guess_remderiv);
 	    options.add(temporal);
+		options.add(retryfactor);
+		options.add(econ);
      }
      catch(X_OptionError& e) {
        options.usage();
